/* src/vm/jit/arm/md-abi.h - defines for arm ABI

   Copyright (C) 1996-2005 R. Grafl, A. Krall, C. Kruegel, C. Oates,
   R. Obermaisser, M. Platter, M. Probst, S. Ring, E. Steiner,
   C. Thalinger, D. Thuernbeck, P. Tomsich, C. Ullrich, J. Wenninger,
   Institut f. Computersprachen - TU Wien

   This file is part of CACAO.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   Contact: cacao@complang.tuwien.ac.at

   Authors: Michael Starzinger

   $Id: md-abi.h,v 1.5 2005/07/09 01:47:28 michi Exp $

*/


#ifndef _MD_ABI_H
#define _MD_ABI_H

/* preallocated registers *****************************************************/

/* integer registers */
#define REG_RESULT      0    /* to deliver method results                     */
#define REG_RESULT2     1    /* to deliver long method results                */

#define REG_IP          12   /* intra-procedure-call scratch register         */
#define REG_SP          13   /* stack pointer                                 */
#define REG_LR          14   /* link register                                 */
#define REG_PC          15   /* program counter                               */

/*#define REG_METHODPTR   12 */  /* pointer to the place from where the procedure */
                             /* vector has been fetched                       */
/* TODO: correct this */
#define REG_METHODPTR REG_ITMP2
#define REG_ITMP1       10   /* temporary register                            */
#define REG_ITMP2       11   /* temporary register                            */
#define REG_ITMP3       9    /* temporary register and method pointer         */
#define REG_ITMP1_XPTR  10   /* exception pointer = temporary register 1      */
#define REG_ITMP2_XPC   11   /* exception pc = temporary register 2           */


/* floating point registers */
/*#define REG_FRESULT     0*/    /* to deliver floating point method results      */
#define REG_FTMP1       6    /* temporary floating point register             */
#define REG_FTMP2       7    /* temporary floating point register             */
/*#define REG_FTMP3       3*/    /* temporary floating point register             */
/*#define REG_IFTMP       0*/    /* temporary integer and floating point register */


/* register count */
#define INT_REG_CNT     16
#define INT_TMP_CNT     0
#define INT_SAV_CNT     5   /* number of int callee saved registers          */
#define INT_ARG_CNT     4   /* number of int argument registers              */

/* TODO: fix this */
#define FLT_REG_CNT     8
#define FLT_TMP_CNT     6
#define FLT_SAV_CNT     0   /* number of flt callee saved registers          */
#define FLT_ARG_CNT     0   /* number of flt argument registers              */

#define TRACE_ARGS_NUM  4


/* Register Pack/Unpack Macros ************************************************/
#define GET_LOW_REG(a)  (((a) & 0xffff0000) >> 16)
#define GET_HIGH_REG(a) ((a) &  0x0000ffff)
#define PACK_REGS(low,high) \
	( ((high) & 0x0000ffff) | (((low) & 0x0000ffff) << 16) )

#define ARGUMENT_REGS(t,a) \
	(IS_2_WORD_TYPE(t)) ? \
	   (PACK_REGS(rd->argintregs[GET_LOW_REG(a)],rd->argintregs[GET_HIGH_REG(a)])) : \
	   (rd->argintregs[(a)])


#endif /* _MD_ABI_H */


/*
 * These are local overrides for various environment variables in Emacs.
 * Please do not remove this and leave it at the end of the file, where
 * Emacs will automagically detect them.
 * ---------------------------------------------------------------------
 * Local variables:
 * mode: c
 * indent-tabs-mode: t
 * c-basic-offset: 4
 * tab-width: 4
 * End:
 */
