/* src/vm/jit/arm/patcher.c - arm code patching functions

   Copyright (C) 1996-2005 R. Grafl, A. Krall, C. Kruegel, C. Oates,
   R. Obermaisser, M. Platter, M. Probst, S. Ring, E. Steiner,
   C. Thalinger, D. Thuernbeck, P. Tomsich, C. Ullrich, J. Wenninger,
   Institut f. Computersprachen - TU Wien

   This file is part of CACAO.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   Contact: cacao@complang.tuwien.ac.at

   Authors: Michael Starzinger

   $Id: patcher.c,v 1.8 2005/07/10 19:22:41 michi Exp $

*/


#include "vm/jit/arm/types.h"

#include "mm/memory.h"
#include "native/native.h"
#include "vm/builtin.h"
#include "vm/field.h"
#include "vm/initialize.h"
#include "vm/options.h"
#include "vm/references.h"
#include "vm/jit/asmpart.h"
#include "vm/jit/helper.h"
#include "vm/jit/patcher.h"

#define gen_getoffset(offset,inst) \
    (offset) = (s2) (inst) & 0x00000fff; \
    (offset) *= (-1);
/* ATTENTION we assume that the offset of LDR and STR is always negative */
/* TODO: maybe correct this ugly hack sometime */

#define gen_resolveload(inst,offset) \
	if ((offset) <  0) { \
		(inst) |= (-(offset)) & 0x0fff; \
		(inst) &= ~(1 << 23); \
	} else { \
		(inst) |= (offset) & 0x0fff; \
		/*(inst) |= (1 << 23); */ \
	}


/* patcher_get_putstatic *******************************************************

   Machine code:

   <patched call position>
   e51c103c    ldr   r1, [ip, #-60]

*******************************************************************************/

bool patcher_get_putstatic(u1 *sp)
{
	u1                *ra;
	java_objectheader *o;
	u4                 mcode;
	unresolved_field  *uf;
	u1                *pv;
	fieldinfo         *fi;
	s2                 offset;

	/*dolog("patching: get_putstatic at %p", sp);*/

	/* get stuff from the stack */
	ra    = (u1 *)                *((ptrint *) (sp + 3 * 4));
	o     = (java_objectheader *) *((ptrint *) (sp + 2 * 4));
	mcode =                       *((u4 *)     (sp + 1 * 4));
	uf    = (unresolved_field *)  *((ptrint *) (sp + 0 * 4));
	pv    = (u1 *)                *((ptrint *) (sp - 1 * 4));
	/*dolog("patching: ra=%p, o=%p, mcode=%lx, uf=%p, pv=%p", ra, o, mcode, uf, pv);*/

	/* calculate and set the new return address */
	ra = ra - 4;
	*((ptrint*) (sp + 3 * 4)) = (ptrint) ra;

	PATCHER_MONITORENTER;

	/* get the fieldinfo */
	if (!(fi = helper_resolve_fieldinfo(uf))) {
		PATCHER_MONITOREXIT;
		return false;
	}

	/* check if the field's class is initialized */
	if (!fi->class->initialized) {
		if (!initialize_class(fi->class)) {
			PATCHER_MONITOREXIT;
			return false;
		}
	}

	/* patch back original code */
	*((u4*) ra) = mcode;

	/* if we show disassembly, we have to skip the nop */
	if (opt_showdisassemble)
		ra = ra + 4;

	/* get the offset from machine instruction */
	gen_getoffset(offset, *((u4*) ra));
	/*dolog("patching: offset=%d, field_value_addr=%p", offset, &(fi->value));*/

	/* patch the field value's address */
	*((ptrint*) (pv + offset)) = (ptrint) &(fi->value);

	/* synchronize instruction cache */
	asm_sync_instruction_cache();

	PATCHER_MARK_PATCHED_MONITOREXIT;

	return true;
}


/* patcher_get_putfield ********************************************************

   Machine code:

   <patched call position>
   e58a8000    str   r8, [sl, #__]

*******************************************************************************/

bool patcher_get_putfield(u1 *sp)
{
	u1                *ra;
	java_objectheader *o;
	u4                 mcode;
	unresolved_field  *uf;
	u1                *pv;
	fieldinfo         *fi;

	/*dolog("patching: get_putfield at %p", sp);*/

	ra    = (u1*)                 *((ptrint *) (sp + 3 * 4));
	o     = (java_objectheader*)  *((ptrint *) (sp + 2 * 4));
	mcode =                       *((u4 *)     (sp + 1 * 4));
	uf    = (unresolved_field*)   *((ptrint *) (sp + 0 * 4));
	pv    = (u1*)                 *((ptrint *) (sp - 1 * 4));
	/*dolog("patching: ra=%p, o=%p, mcode=%lx, uf=%p, pv=%p", ra, o, mcode, uf, pv);*/

	/* calculate and set the new return address */
	ra = ra - 4;
	*((ptrint*) (sp + 3 * 4)) = (ptrint) ra;

	PATCHER_MONITORENTER;

	/* get the fieldinfo */
	if (!(fi = helper_resolve_fieldinfo(uf))) {
		PATCHER_MONITOREXIT;
		return false;
	}

	/* patch back original code */
	*((u4*) ra) = mcode;

	/* if we show disassembly, we have to skip the nop */
	if (opt_showdisassemble)
		ra = ra + 4;

	/* patch the field's offset */
    /*dolog("patching: old=%lx offset=%d", *((u4*) ra), fi->offset);*/
    gen_resolveload(*((u4*) ra), fi->offset);
	/*dolog("patching: new=%lx", *((u4*) ra));*/

	/* synchronize instruction cache */
	asm_sync_instruction_cache();

	PATCHER_MARK_PATCHED_MONITOREXIT;

	return true;
}


/* patcher_builtin_new *********************************************************

   Machine code:

   e51c002c    ldr   r0, [ip, #-44]
   <patched call postition>
   e51cc030    ldr   ip, [ip, #-48]
   e1a0e00f    mov   lr, pc
   e1a0f00c    mov   pc, ip

*******************************************************************************/

bool patcher_builtin_new(u1 *sp)
{
	u1                *ra;
	java_objectheader *o;
	u4                 mcode;
	constant_classref *cr;
	u1                *pv;
	classinfo         *c;
	s2                offset;

	/*dolog("patching: builtin_new at %p", sp);*/

	/* get stuff from the stack */
	ra    = (u1 *)                *((ptrint *) (sp + 3 * 4));
	o     = (java_objectheader *) *((ptrint *) (sp + 2 * 4));
	mcode =                       *((u4 *)     (sp + 1 * 4));
	cr    = (constant_classref *) *((ptrint *) (sp + 0 * 4));
	pv    = (u1 *)                *((ptrint *) (sp - 1 * 4));
	/*dolog("patching: ra=%p, o=%p, mcode=%lx, cr=%p, pv=%p", ra, o, mcode, cr, pv);*/

	/* calculate and set the new return address */
	ra = ra - (4 + 4);
	*((ptrint*) (sp + 3 * 4)) = (ptrint) ra;

	PATCHER_MONITORENTER;

	/* get the classinfo */
	if (!(c = helper_resolve_classinfo(cr))) {
		PATCHER_MONITOREXIT;
		return false;
	}

	/* patch back original code */
	*((u4*) (ra + 4)) = mcode;

	/* get the offset from machine instruction */
	gen_getoffset(offset, *((u4*) ra));
	/*{ char txt[255]; utf_sprint(txt,c->name); dolog("patching: offset=%d, classinfo=%p (%s)", offset, c, txt); }*/

	/* patch the classinfo pointer */
	*((ptrint*) (pv + offset)) = (ptrint) c;

	/* if we show disassembly, we have to skip the nop */
	if (opt_showdisassemble)
		ra = ra + 4;

	/* get the offset from machine instruction */
	gen_getoffset(offset, *((u4*) (ra + 4)));
	/*dolog("patching: offset=%d, builtin_new=%p", offset, BUILTIN_new);*/

	/* patch new function address */
	*((ptrint*) (pv + offset)) = (ptrint) BUILTIN_new;

	/* synchronize instruction cache */
	asm_sync_instruction_cache();

	PATCHER_MARK_PATCHED_MONITOREXIT;

	return true;
}


/* patcher_builtin_newarray ****************************************************

   Machine code:

   e51c002c    ldr   r0, [ip, #-44]
   <patched call postition>
   e51cc030    ldr   ip, [ip, #-48]
   e1a0e00f    mov   lr, pc
   e1a0f00c    mov   pc, ip

*******************************************************************************/

bool patcher_builtin_newarray(u1 *sp)
{
	u1                *ra;
	java_objectheader *o;
	u4                 mcode;
	constant_classref *cr;
	u1                *pv;
	classinfo         *c;
	s2                 offset;

	/*dolog("patching: builtin_newarray at %p", sp);*/

	/* get stuff from the stack */
	ra    = (u1*)                 *((ptrint *) (sp + 3 * 4));
	o     = (java_objectheader*)  *((ptrint *) (sp + 2 * 4));
	mcode =                       *((u4 *)     (sp + 1 * 4));
	cr    = (constant_classref*)  *((ptrint *) (sp + 0 * 4));
	pv    = (u1 *)                *((ptrint *) (sp - 1 * 4));
	/*dolog("patching: ra=%p, o=%p, mcode=%lx, cr=%p, pv=%p", ra, o, mcode, cr, pv);*/

	/* calculate and set the new return address */
	ra = ra - (4 + 4);
	*((ptrint*) (sp + 3 * 4)) = (ptrint) ra;

	PATCHER_MONITORENTER;

	/* get the classinfo */
	if (!(c = helper_resolve_classinfo(cr))) {
		PATCHER_MONITOREXIT;
		return false;
	}

	/* patch back original code */
	*((u4 *) (ra + 4)) = mcode;

	/* get the offset from machine instruction */
	gen_getoffset(offset, *((u4*) ra));
	/*dolog("patching: offset=%d, vftbl=%p", offset, (ptrint) c->vftbl);*/

	/* patch the class' vftbl pointer */
	*((ptrint*) (pv + offset)) = (ptrint) c->vftbl;

	/* if we show disassembly, we have to skip the nop */
	if (opt_showdisassemble)
		ra = ra + 4;

	/* get the offset from machine instruction */
	gen_getoffset(offset, *((u4*) (ra + 4)));
	/*dolog("patching: offset=%d, builtin_newarray=%p", offset, BUILTIN_newarray);*/

	/* patch new function address */
	*((ptrint*) (pv + offset)) = (ptrint) BUILTIN_newarray;

	/* synchronize instruction cache */
	asm_sync_instruction_cache();

	PATCHER_MARK_PATCHED_MONITOREXIT;

	return true;
}


/* patcher_builtin_multianewarray **********************************************

   Machine code:

   <patched call position>
   221f0002    lda     a0,2(zero)
   a63bff80    ldq     a1,-128(pv)
   47de0412    mov     sp,a2
   a77bff78    ldq     pv,-136(pv)
   6b5b4000    jsr     (pv)

*******************************************************************************/

bool patcher_builtin_multianewarray(u1 *sp)
{
	dolog("patching: builtin_multianewarray at %p", sp);
	return false;
}


/* patcher_builtin_arraycheckcast **********************************************

   Machine code:

   a63bfe60    ldq     a1,-416(pv)
   <patched call position>
   a77bfe58    ldq     pv,-424(pv)
   6b5b4000    jsr     (pv)

*******************************************************************************/

bool patcher_builtin_arraycheckcast(u1 *sp)
{
	dolog("patching: builtin_arraycheckcast at %p", sp);
	return false;
}


/* patcher_builtin_arrayinstanceof *********************************************

   Machine code:

   a63bfeb0    ldq     a1,-336(pv)
   <patched call position>
   a77bfea8    ldq     pv,-344(pv)
   6b5b4000    jsr     (pv)

*******************************************************************************/

bool patcher_builtin_arrayinstanceof(u1 *sp)
{
	dolog("patching: builtin_arrayinstanceof at %p", sp);
	return false;
}


/* patcher_invokestatic_special ************************************************

   Machine code:

   <patched call position>
   e51cc02c    ldr   ip, [ip, #-44]
   e1a0e00f    mov   lr, pc
   e1a0f00c    mov   pc, ip

******************************************************************************/

bool patcher_invokestatic_special(u1 *sp)
{
	u1                *ra;
	java_objectheader *o;
	u4                 mcode;
	unresolved_method *um;
	u1                *pv;
	methodinfo        *m;
	s4                 offset;

	/*dolog("patching: invokestatic_special at %p", sp);*/

	/* get stuff from the stack */
	ra    = (u1*)                 *((ptrint *) (sp + 3 * 4));
	o     = (java_objectheader*)  *((ptrint *) (sp + 2 * 4));
	mcode =                       *((u4 *)     (sp + 1 * 4));
	um    = (unresolved_method*)  *((ptrint *) (sp + 0 * 4));
	pv    = (u1*)                 *((ptrint *) (sp - 1 * 4));
	/*dolog("patching: ra=%p, o=%p, mcode=%lx, um=%p, pv=%p", ra, o, mcode, um, pv);*/

	/* calculate and set the new return address */
	ra = ra - 4;
	*((ptrint*) (sp + 3 * 4)) = (ptrint) ra;

	PATCHER_MONITORENTER;

	/* get the fieldinfo */
	if (!(m = helper_resolve_methodinfo(um))) {
		PATCHER_MONITOREXIT;
		return false;
	}

	/* patch back original code */
	*((u4*) ra) = mcode;

	/* if we show disassembly, we have to skip the nop */
	if (opt_showdisassemble)
		ra = ra + 4;

	/* get the offset from machine instruction */
	gen_getoffset(offset, *((u4*) ra));
	/*dolog("patching: offset=%d, stubroutine=%p", offset, m->stubroutine);*/

	/* patch stubroutine */
	*((ptrint*) (pv + offset)) = (ptrint) m->stubroutine;

	/* synchronize instruction cache */
	asm_sync_instruction_cache();

	PATCHER_MARK_PATCHED_MONITOREXIT;

	return true;
}


/* patcher_invokevirtual *******************************************************

   Machine code:

   <patched call position>
   e590b000    ldr   fp, [r0]
   e59bc000    ldr   ip, [fp, #__]
   e1a0e00f    mov   lr, pc
   e1a0f00c    mov   pc, ip

*******************************************************************************/

bool patcher_invokevirtual(u1 *sp)
{
	u1                *ra;
	java_objectheader *o;
	u4                 mcode;
	unresolved_method *um;
	methodinfo        *m;

	/*dolog("patching: invokevirtual at %p", sp);*/

	/* get stuff from the stack */
	ra    = (u1*)                 *((ptrint *) (sp + 3 * 4));
	o     = (java_objectheader*)  *((ptrint *) (sp + 2 * 4));
	mcode =                       *((u4 *)     (sp + 1 * 4));
	um    = (unresolved_method*)  *((ptrint *) (sp + 0 * 4));
	/*dolog("patching: ra=%p, o=%p, mcode=%lx, um=%p", ra, o, mcode, um);*/

	/* calculate and set the new return address */
	ra = ra - 4;
	*((ptrint*) (sp + 3 * 4)) = (ptrint) ra;

	PATCHER_MONITORENTER;

	/* get the fieldinfo */
	if (!(m = helper_resolve_methodinfo(um))) {
		PATCHER_MONITOREXIT;
		return false;
	}

	/* patch back original code */
	*((u4*) ra) = mcode;

	/* if we show disassembly, we have to skip the nop */
	if (opt_showdisassemble)
		ra = ra + 4;

	/* patch vftbl index */
	/*dolog("patching: old=%lx offset=%d", *((s4*) (ra + 4)), (OFFSET(vftbl_t, table[0]) + sizeof(methodptr) * m->vftblindex));*/
	gen_resolveload(*((s4*) (ra + 4)),  (s4) (OFFSET(vftbl_t, table[0]) + sizeof(methodptr) * m->vftblindex));
	/*dolog("patching: new=%lx", *((s4*) (ra + 4)));*/

	/* synchronize instruction cache */
	asm_sync_instruction_cache();

	PATCHER_MARK_PATCHED_MONITOREXIT;

	return true;
}


/* patcher_invokeinterface *****************************************************

   Machine code:

   <patched call position>
   e590b000    ldr   fp, [r0]
   e59bb000    ldr   fp, [fp, #__]
   e59bc000    ldr   ip, [fp, #__]
   e1a0e00f    mov   lr, pc
   e1a0f00c    mov   pc, ip


*******************************************************************************/

bool patcher_invokeinterface(u1 *sp)
{
	u1                *ra;
	java_objectheader *o;
	u4                 mcode;
	unresolved_method *um;
	methodinfo        *m;

	/*dolog("patching: invokeinterface at %p", sp);*/

	/* get stuff from the stack */
	ra    = (u1*)                 *((ptrint *) (sp + 3 * 4));
	o     = (java_objectheader*)  *((ptrint *) (sp + 2 * 4));
	mcode =                       *((u4 *)     (sp + 1 * 4));
	um    = (unresolved_method*)  *((ptrint *) (sp + 0 * 4));
	/*dolog("patching: ra=%p, o=%p, mcode=%lx, um=%p", ra, o, mcode, um);*/

	/* calculate and set the new return address */
	ra = ra - 4;
	*((ptrint*) (sp + 3 * 4)) = (ptrint) ra;

	PATCHER_MONITORENTER;

	/* get the fieldinfo */
	if (!(m = helper_resolve_methodinfo(um))) {
		PATCHER_MONITOREXIT;
		return false;
	}

	/* patch back original code */
	*((u4*) ra) = mcode;

	/* if we show disassembly, we have to skip the nop */
	if (opt_showdisassemble)
		ra = ra + 4;

	/* patch interfacetable index */
	/*dolog("patching: old=%lx offset=%d", *((s4*) (ra + 4)), (OFFSET(vftbl_t, table[0]) - sizeof(methodptr*) * m->class->index));*/
	gen_resolveload(*((s4*) (ra + 4)), (s4) (OFFSET(vftbl_t, interfacetable[0]) - sizeof(methodptr*) * m->class->index));
	/*dolog("patching: new=%lx", *((s4*) (ra + 4)));*/

	/* patch method offset */
	/*dolog("patching: old=%lx offset=%d", *((s4*) (ra + 4 + 4)), (sizeof(methodptr) * (m - m->class->methods)));*/
	gen_resolveload(*((s4*) (ra + 4 + 4)), (s4) (sizeof(methodptr) * (m - m->class->methods)));
	/*dolog("patching: new=%lx", *((s4*) (ra + 4 + 4)));*/

	/* synchronize instruction cache */
	asm_sync_instruction_cache();

	PATCHER_MARK_PATCHED_MONITOREXIT;

	return true;
}


/* patcher_checkcast_instanceof_flags ******************************************

   Machine code:

   <patched call position>

*******************************************************************************/

bool patcher_checkcast_instanceof_flags(u1 *sp)
{
	dolog("patching: checkcast_instanceof_flags at %p", sp);
	return false;
}


/* patcher_checkcast_instanceof_interface **************************************

   Machine code:

   <patched call position>
   a78e0000    ldq     at,0(s5)
   a3bc001c    ldl     gp,28(at)
   23bdfffd    lda     gp,-3(gp)
   efa0002e    ble     gp,0x00000200002bf6b0
   a7bcffe8    ldq     gp,-24(at)

*******************************************************************************/

bool patcher_checkcast_instanceof_interface(u1 *sp)
{
	dolog("patching: checkcast_instanceof_interface at %p", sp);
	return false;
}


/* patcher_checkcast_instanceof_class ******************************************

   Machine code:

   <patched call position>
   a7940000    ldq     at,0(a4)
   a7bbff28    ldq     gp,-216(pv)

*******************************************************************************/

bool patcher_checkcast_instanceof_class(u1 *sp)
{
	dolog("patching: checkcast_instanceof_class at %p", sp);
	return false;
}


/* patcher_clinit **************************************************************

   XXX

*******************************************************************************/

bool patcher_clinit(u1 *sp)
{
	u1                *ra;
	java_objectheader *o;
	u4                 mcode;
	classinfo         *c;

	/*dolog("patching: clinit at %p", sp);*/

	/* get stuff from the stack */
	ra    = (u1 *)                *((ptrint *) (sp + 3 * 4));
	o     = (java_objectheader *) *((ptrint *) (sp + 2 * 4));
	mcode =                       *((u4 *)     (sp + 1 * 4));
	c     = (classinfo *)         *((ptrint *) (sp + 0 * 4));
	/*dolog("patching: ra=%p, o=%p, mcode=%lx, c=%p", ra, o, mcode, c);*/

	/* calculate and set the new return address */
	ra = ra - 4;
	*((ptrint*) (sp + 3 * 4)) = (ptrint) ra;

	PATCHER_MONITORENTER;

	/* check if the class is initialized */
	if (!c->initialized) {
		if (!initialize_class(c)) {
			PATCHER_MONITOREXIT;
			return false;
		}
	}

	/* patch back original code */
	*((u4*) ra) = mcode;

	/* synchronize instruction cache */
	asm_sync_instruction_cache();

	PATCHER_MARK_PATCHED_MONITOREXIT;

	return true;
}


/* patcher_resolve_native ******************************************************

   XXX

*******************************************************************************/

#if !defined(ENABLE_STATICVM)
bool patcher_resolve_native(u1 *sp)
{
	u1                *ra;
	java_objectheader *o;
	u4                 mcode;
	methodinfo        *m;
	u1                *pv;
	functionptr        f;
	s2                 offset;

	dolog("patching: resolve_native at %p", sp);

	/* get stuff from the stack */
	ra    = (u1 *)                *((ptrint *) (sp + 3 * 4));
	o     = (java_objectheader *) *((ptrint *) (sp + 2 * 4));
	mcode =                       *((u4 *)     (sp + 1 * 4));
	m     = (methodinfo *)        *((ptrint *) (sp + 0 * 4));
	pv    = (u1 *)                *((ptrint *) (sp - 1 * 4));
	dolog("patching: ra=%p, o=%p, mcode=%lx, m=%p, pv=%p", ra, o, mcode, m, pv);
	dolog("patching: native function %s.%s", m->class->name->text, m->name->text);

	/* calculate and set the new return address */
	ra = ra - 4;
	*((ptrint*) (sp + 3 * 4)) = (ptrint) ra;

	PATCHER_MONITORENTER;

	/* resolve native function */
	if (!(f = native_resolve_function(m))) {
		dolog("patching: no functionptr for %s", m->name->text);
		PATCHER_MONITOREXIT;
		return false;
	}
	dolog("patching: got functionptr (f=%p) for %s", f, m->name->text);

	/* patch back original code */
	*((u4*) ra) = mcode;

	/* if we show disassembly, we have to skip the nop */
	if (opt_showdisassemble)
		ra = ra + 4;

	/* get the offset from machine instruction */
	gen_getoffset(offset, *((u4*) (ra + 4)));
	dolog("patching: offset=%d, functionptr=%p", offset, f);

	/* patch native function pointer */
	*((ptrint*) (pv + offset)) = (ptrint) f;

	/* synchronize instruction cache */
	asm_sync_instruction_cache();

	PATCHER_MARK_PATCHED_MONITOREXIT;

	return true;
}
#endif /* !defined(ENABLE_STATICVM) */


/*
 * These are local overrides for various environment variables in Emacs.
 * Please do not remove this and leave it at the end of the file, where
 * Emacs will automagically detect them.
 * ---------------------------------------------------------------------
 * Local variables:
 * mode: c
 * indent-tabs-mode: t
 * c-basic-offset: 4
 * tab-width: 4
 * End:
 * vim:noexpandtab:sw=4:ts=4:
 */
